@@ -46,8 +46,8 @@
 #'
 #' @param Pedigree reference pedigree from which to simulate, dataframe with
 #'   columns id-dam-sire. Additional columns are ignored.
-#' @param LifeHistData dataframe with id, sex (1=female, 2=male, 3=unknown), and
-#'   birth year.
+#' @param LifeHistData dataframe with id, sex (1=female, 2=male, 3=unknown),
+#' birth year, and optionally BY.min - BY.max - YearLast.
 #' @param args.sim  list of arguments to pass to \code{\link{SimGeno}}, such as
 #'   \code{nSnp} (number of SNPs), \code{SnpError} (genotyping error rate) and
 #'   \code{ParMis} (proportion of non-genotyped parents). Set to \code{NULL} to
@@ -121,15 +121,19 @@
 #' sumry_grif <- SummarySeq(SeqOUT_griffin, Plot=FALSE)
 #' tmp <- apply(sumry_grif$ParentCount['Genotyped',,,],
 #'              MARGIN = c('parentSex', 'parentCat'), FUN = sum)
-#' sweep(tmp, MARGIN='parentCat', STATS = rowSums(tmp), FUN = '/')
+#' props <- sweep(tmp, MARGIN='parentCat', STATS = rowSums(tmp), FUN = '/')
+#' 1 - props[,'Genotyped'] / (props[,'Genotyped'] + props[,'Dummy'])
 #'
 #' # Example for parentage assignment only
 #' conf_grif <- EstConf(Pedigree = SeqOUT_griffin$Pedigree,
 #'                LifeHistData = SeqOUT_griffin$LifeHist,
-#'                args.sim = list(nSnp = 100, SnpError = 5e-3, CallRate=0.8,
-#'                                ParMis=c(0.54, 0.44)),
-#'                args.seq = list(Err=5e-3, Module="par"),
-#'                nSim = 2, nCores=1)
+#'                args.sim = list(nSnp = 200,   # no. in actual data, or what-if
+#'                                SnpError = 5e-3,  # best estimate, or what-if
+#'                                CallRate=0.8,     # from SnpStats()
+#'                                ParMis=c(0.39, 0.20)),  # calc'd above
+#'                args.seq = list(Err=5e-3, Module="par"),  # as in real run
+#'                nSim = 2,   # try-out, proper run >=20 (10 if huge pedigree)
+#'                nCores=1)
 #'
 #' # parent-pair confidence, per category (Genotyped/Dummy/None)
 #' conf_grif$ConfProb
@@ -195,7 +199,7 @@ EstConf <- function(Pedigree = NULL,
     args.sim[["Err"]] <- NULL    # common confusion, otherwise fuzy matching with 'ErrorFM'.
   }
 
-  Ped.ref <- Pedigree[,1:3]
+  Ped.ref <- PedPolish(Pedigree, KeepAllColumns=FALSE)
   if (any(substr(unlist(Ped.ref),1,6) %in% c("sim_F0", "sim_M0"))) {
     stop("Please don't use 'sim_F' or 'sim_M' in reference pedigree")
   }
@@ -279,13 +283,14 @@ EstConf <- function(Pedigree = NULL,
   if (nCores>1) {
     cl <- parallel::makeCluster(nCores)
     AllOUT <- parallel::parLapply(cl, X=seq.int(nSim), fun=SimInfer,
-                                  RefPedigree = Pedigree,
+                                  RefPedigree = Ped.ref,
                                   args.sim, LifeHistData, args.seq,
                                   quiet.EC=TRUE, ParSib)
     #                                  chunk.size=1)
     parallel::stopCluster(cl)
   } else {
-    AllOUT <- plyr::llply(seq.int(nSim), SimInfer, RefPedigree = Pedigree,
+    AllOUT <- plyr::llply(seq.int(nSim), .fun=SimInfer,
+                          RefPedigree = Ped.ref,
                           args.sim, LifeHistData, args.seq,
                           quiet.EC, ParSib)
   }
